@@ -175,6 +175,11 @@ public:
    */
   pair<int, int> const & decode(int const code) const;
 
+  /**
+   * Get all symbols where the left-hand side of the symbol-pair is l.
+   */
+  set<int> symbolsWhereLeftIs(wchar_t l) const;
+
   enum Side
   {
     left,
