@@ -44,6 +44,7 @@ wstring const Compiler::COMPILER_PAIR_ELEM          = L"p";
 wstring const Compiler::COMPILER_LEFT_ELEM          = L"l";
 wstring const Compiler::COMPILER_RIGHT_ELEM         = L"r";
 wstring const Compiler::COMPILER_S_ELEM             = L"s";
+wstring const Compiler::COMPILER_M_ELEM             = L"m";
 wstring const Compiler::COMPILER_REGEXP_ELEM        = L"re";
 wstring const Compiler::COMPILER_SECTION_ELEM       = L"section";
 wstring const Compiler::COMPILER_ID_ATTR            = L"id";
@@ -128,8 +129,35 @@ Compiler::parse(string const &file, wstring const &dir)
   {
     it.second.minimize();
   }
+
+  if (!valid(dir)) {
+    exit(EXIT_FAILURE);
+  }
 }
 
+bool
+Compiler::valid(wstring const& dir) const
+{
+  const wstring side = dir == COMPILER_RESTRICTION_RL_VAL ? L"right" : L"left";
+  const set<int> epsilonSymbols = alphabet.symbolsWhereLeftIs(0);
+  const set<int> spaceSymbols = alphabet.symbolsWhereLeftIs(L' ');
+  for (auto &section : sections) {
+    auto &fst = section.second;
+    auto finals = fst.getFinals();
+    auto initial = fst.getInitial();
+    for(const auto i : fst.closure(initial, epsilonSymbols)) {
+      if (finals.count(i)) {
+        wcerr << L"Error: Invalid dictionary (hint: the " << side << " side of an entry is empty)" << endl;
+        return false;
+      }
+      if(fst.closure(i, spaceSymbols).size() > 1) { // >1 since closure always includes self
+        wcerr << L"Error: Invalid dictionary (hint: entry on the " << side << " beginning with whitespace)" << endl;
+        return false;
+      }
+    }
+  }
+  return true;
+}
 
 void
 Compiler::procAlphabet()
@@ -320,6 +348,14 @@ Compiler::readString(list<int> &result, wstring const &name)
       result.push_back(static_cast<int>(value[i]));
     }
   }
+  else if(name == COMPILER_M_ELEM)
+  {
+    requireEmptyError(name);
+    if(keep_boundaries)
+    {
+      result.push_back(static_cast<int>(L'>'));
+    }
+  }
   else if(name == COMPILER_BLANK_ELEM)
   {
     requireEmptyError(name);
@@ -959,6 +995,12 @@ Compiler::setVariantRightValue(string const &v)
   variant_right = XMLParseUtil::stows(v);
 }
 
+void
+Compiler::setKeepBoundaries(bool keep)
+{
+  keep_boundaries = keep;
+}
+
 void
 Compiler::setVerbose(bool verbosity)
 {
