@@ -25,7 +25,7 @@
 #include <iostream>
 #include <libxml/encoding.h>
 
-#ifdef _WIN32
+#if defined(_WIN32) && !defined(_MSC_VER)
 #include <utf8_fwrap.h>
 #endif
 
@@ -113,6 +113,13 @@ Expander::readString(wstring &result, wstring const &name)
   if(name == L"#text")
   {
     wstring value = XMLParseUtil::towstring(xmlTextReaderConstValue(reader));
+    wstring escaped = L"^$/<>{}\\*@#+~:";
+    for(size_t i = value.size()-1; i > 0; i--)
+    {
+      if(escaped.find(value[i]) != wstring::npos) {
+        value.insert(value.begin()+i, L'\\');
+      }
+    }
     result.append(value);
   }
   else if(name == Compiler::COMPILER_BLANK_ELEM)
@@ -120,6 +127,14 @@ Expander::readString(wstring &result, wstring const &name)
     requireEmptyError(name);
     result += L' ';
   }
+  else if(name == Compiler::COMPILER_M_ELEM)
+  {
+    requireEmptyError(name);
+    if(keep_boundaries)
+    {
+      result += L'>';
+    }
+  }
   else if(name == Compiler::COMPILER_JOIN_ELEM)
   {
     requireEmptyError(name);
@@ -335,7 +350,7 @@ Expander::procEntry(FILE *output)
 
   wstring myname = L"";
   if(this->attrib(Compiler::COMPILER_IGNORE_ATTR) == L"yes"
-   || altval != L"" && altval != alt
+   || (altval != L"" && altval != alt)
    || (varval != L"" && varval != variant && attribute == Compiler::COMPILER_RESTRICTION_RL_VAL)
    || ((varl != L"" && varl != variant_left) && (varr != L"" && varr != variant_right))
    || (varl != L"" && varl != variant_left && attribute == Compiler::COMPILER_RESTRICTION_RL_VAL)
@@ -359,7 +374,7 @@ Expander::procEntry(FILE *output)
   EntList items, items_lr, items_rl;
   if(attribute == Compiler::COMPILER_RESTRICTION_LR_VAL
    || (varval != L"" && varval != variant && attribute != Compiler::COMPILER_RESTRICTION_RL_VAL)
-   || varl != L"" && varl != variant_left)
+   || (varl != L"" && varl != variant_left))
   {
     items_lr.push_back(make_pair(L"", L""));
   }
@@ -659,3 +674,10 @@ Expander::setVariantRightValue(string const &v)
 {
   variant_right = XMLParseUtil::stows(v);
 }
+
+void
+Expander::setKeepBoundaries(bool keep)
+{
+  keep_boundaries = keep;
+}
+
