@@ -22,8 +22,9 @@
 #include <iostream>
 #include <cerrno>
 #include <climits>
+#include <cwctype>
 
-#ifdef _WIN32
+#if defined(_WIN32) && !defined(_MSC_VER)
 #include <utf8_fwrap.h>
 #endif
 
@@ -200,7 +201,7 @@ FSTProcessor::readEscaped(FILE *input)
 
   wchar_t val = static_cast<wchar_t>(fgetwc_unlocked(input));
 
-  if(feof(input) || escaped_chars.find(val) == escaped_chars.end())
+  if(feof(input))
   {
     streamError();
   }
@@ -237,6 +238,96 @@ FSTProcessor::readFullBlock(FILE *input, wchar_t const delim1, wchar_t const del
   return result;
 }
 
+wstring
+FSTProcessor::readWblank(FILE *input)
+{
+  wstring result = L"";
+  result += L"[[";
+  wchar_t c = 0;
+
+  while(!feof(input))
+  {
+    c = static_cast<wchar_t>(fgetwc_unlocked(input));
+    result += c;
+
+    if(c == L'\\')
+    {
+      result += static_cast<wchar_t>(readEscaped(input));
+    }
+    else if(c == L']')
+    {
+      c = static_cast<wchar_t>(fgetwc_unlocked(input));
+      result += c;
+
+      if(c == L']')
+      {
+        break;
+      }
+    }
+  }
+
+  if(c != L']')
+  {
+    streamError();
+  }
+
+  return result;
+}
+
+bool
+FSTProcessor::wblankPostGen(FILE *input, FILE *output)
+{
+  wstring result = L"";
+  result += L"[[";
+  wchar_t c = 0;
+
+  while(!feof(input))
+  {
+    c = static_cast<wchar_t>(fgetwc_unlocked(input));
+    result += c;
+
+    if(c == L'\\')
+    {
+      result += static_cast<wchar_t>(readEscaped(input));
+    }
+    else if(c == L']')
+    {
+      c = static_cast<wchar_t>(fgetwc_unlocked(input));
+      result += c;
+
+      if(c == L']')
+      {
+        int resultlen = result.size();
+        if(result[resultlen-5] == '[' && result[resultlen-4] == '[' && result[resultlen-3] == '/') //ending blank [[/]]
+        {
+          fputws(result.c_str(), output);
+          break;
+        }
+        else
+        {
+          c = static_cast<wchar_t>(fgetwc_unlocked(input));
+          if(c == L'~')
+          {
+            wblankqueue.push(result);
+            return true;
+          }
+          else
+          {
+            result += c;
+          }
+        }
+      }
+    }
+  }
+
+  if(c != L']')
+  {
+    streamError();
+  }
+
+  return false;
+}
+
 int
 FSTProcessor::readAnalysis(FILE *input)
 {
@@ -249,6 +340,7 @@ FSTProcessor::readAnalysis(FILE *input)
   int altval = 0;
   if(feof(input))
   {
+    input_buffer.add(0);        // so it's treated like the NUL byte
     return 0;
   }
 
@@ -268,16 +360,23 @@ FSTProcessor::readAnalysis(FILE *input)
         return altval;
 
       case L'[':
-        blankqueue.push(readFullBlock(input, L'[', L']'));
+        val = static_cast<wchar_t>(fgetwc_unlocked(input));
+
+        if(val == L'[')
+        {
+          blankqueue.push(readWblank(input));
+        }
+        else
+        {
+          ungetwc_unlocked(val, input);
+          blankqueue.push(readFullBlock(input, L'[', L']'));
+        }
+
         input_buffer.add(static_cast<int>(L' '));
         return static_cast<int>(L' ');
 
       case L'\\':
         val = static_cast<wchar_t>(fgetwc_unlocked(input));
-        if(escaped_chars.find(val) == escaped_chars.end())
-        {
-          streamError();
-        }
         input_buffer.add(static_cast<int>(val));
         return val;
 
@@ -319,17 +418,24 @@ FSTProcessor::readTMAnalysis(FILE *input)
         return altval;
 
       case L'[':
-        blankqueue.push(readFullBlock(input, L'[', L']'));
+        val = static_cast<wchar_t>(fgetwc_unlocked(input));
+
+        if(val == L'[')
+        {
+          blankqueue.push(readWblank(input));
+        }
+        else
+        {
+          ungetwc_unlocked(val, input);
+          blankqueue.push(readFullBlock(input, L'[', L']'));
+        }
+
         input_buffer.add(static_cast<int>(L' '));
         isLastBlankTM = true;
         return static_cast<int>(L' ');
 
       case L'\\':
         val = static_cast<wchar_t>(fgetwc_unlocked(input));
-        if(escaped_chars.find(val) == escaped_chars.end())
-        {
-          streamError();
-        }
         input_buffer.add(static_cast<int>(val));
         return val;
       case L'0':
@@ -349,7 +455,7 @@ FSTProcessor::readTMAnalysis(FILE *input)
             ws += val;
             val = static_cast<wchar_t>(fgetwc_unlocked(input));
           } while(iswdigit(val));
-          ungetwc(val, input);
+          ungetwc_unlocked(val, input);
           input_buffer.add(alphabet(L"<n>"));
           numbers.push_back(ws);
           return alphabet(L"<n>");
@@ -366,7 +472,7 @@ FSTProcessor::readTMAnalysis(FILE *input)
 }
 
 int
-FSTProcessor::readPostgeneration(FILE *input)
+FSTProcessor::readPostgeneration(FILE *input, FILE *output)
 {
   if(!input_buffer.isEmpty())
   {
@@ -375,6 +481,7 @@ FSTProcessor::readPostgeneration(FILE *input)
 
   wchar_t val = static_cast<wchar_t>(fgetwc_unlocked(input));
   int altval = 0;
+  is_wblank = false;
   if(feof(input))
   {
     return 0;
@@ -388,16 +495,37 @@ FSTProcessor::readPostgeneration(FILE *input)
       return altval;
 
     case L'[':
-      blankqueue.push(readFullBlock(input, L'[', L']'));
-      input_buffer.add(static_cast<int>(L' '));
-      return static_cast<int>(L' ');
+      val = static_cast<wchar_t>(fgetwc_unlocked(input));
+
+      if(val == L'[')
+      {
+        if(collect_wblanks)
+        {
+          wblankqueue.push(readWblank(input));
+          is_wblank = true;
+          return static_cast<int>(L' ');
+        }
+        else if(wblankPostGen(input, output))
+        {
+          return static_cast<int>(L'~');
+        }
+        else
+        {
+          is_wblank = true;
+          return static_cast<int>(L' ');
+        }
+      }
+      else
+      {
+        ungetwc_unlocked(val, input);
+        blankqueue.push(readFullBlock(input, L'[', L']'));
+
+        input_buffer.add(static_cast<int>(L' '));
+        return static_cast<int>(L' ');
+      }
 
     case L'\\':
       val = static_cast<wchar_t>(fgetwc_unlocked(input));
-      if(escaped_chars.find(val) == escaped_chars.end())
-      {
-        streamError();
-      }
       input_buffer.add(static_cast<int>(val));
       return val;
 
@@ -515,6 +643,7 @@ FSTProcessor::readGeneration(FILE *input, FILE *output)
   {
     wstring cad = L"";
     cad += static_cast<wchar_t>(val);
+
     while((val = fgetwc_unlocked(input)) != L'>')
     {
       if(feof(input))
@@ -529,7 +658,17 @@ FSTProcessor::readGeneration(FILE *input, FILE *output)
   }
   else if(val == L'[')
   {
-    fputws_unlocked(readFullBlock(input, L'[', L']').c_str(), output);
+    val = fgetwc_unlocked(input);
+    if(val == L'[')
+    {
+      fputws_unlocked(readWblank(input).c_str(), output);
+    }
+    else
+    {
+      ungetwc_unlocked(val, input);
+      fputws_unlocked(readFullBlock(input, L'[', L']').c_str(), output);
+    }
+
     return readGeneration(input, output);
   }
   else
@@ -624,7 +763,17 @@ FSTProcessor::readBilingual(FILE *input, FILE *output)
   }
   else if(val == L'[')
   {
-    fputws_unlocked(readFullBlock(input, L'[', L']').c_str(), output);
+    val = fgetwc_unlocked(input);
+    if(val == L'[')
+    {
+      fputws_unlocked(readWblank(input).c_str(), output);
+    }
+    else
+    {
+      ungetwc_unlocked(val, input);
+      fputws_unlocked(readFullBlock(input, L'[', L']').c_str(), output);
+    }
+
     return readBilingual(input, output);
   }
 
@@ -641,6 +790,59 @@ FSTProcessor::flushBlanks(FILE *output)
   }
 }
 
+void
+FSTProcessor::flushWblanks(FILE *output)
+{
+  while(wblankqueue.size() > 0)
+  {
+    fputws_unlocked(wblankqueue.front().c_str(), output);
+    wblankqueue.pop();
+  }
+}
+
+wstring
+FSTProcessor::combineWblanks()
+{
+  wstring final_wblank;
+  wstring last_wblank = L"";
+
+  while(wblankqueue.size() > 0)
+  {
+    if(wblankqueue.front().compare(L"[[/]]") == 0)
+    {
+      if(final_wblank.empty())
+      {
+        final_wblank += L"[[";
+      }
+      else if(final_wblank.size() > 2)
+      {
+        final_wblank += L"; ";
+      }
+
+      final_wblank += last_wblank.substr(2,last_wblank.size()-4); //add wblank without brackets [[..]]
+      last_wblank.clear();
+    }
+    else
+    {
+      last_wblank = wblankqueue.front();
+    }
+    wblankqueue.pop();
+  }
+
+  if(!last_wblank.empty())
+  {
+    wblankqueue.push(last_wblank);
+  }
+
+  if(!final_wblank.empty())
+  {
+    final_wblank += L"]]";
+    need_end_wblank = true;
+  }
+
+  return final_wblank;
+}
+
 void
 FSTProcessor::calcInitial()
 {
@@ -1297,8 +1499,10 @@ FSTProcessor::analysis(FILE *input, FILE *output)
   bool firstupper = false, uppercase = false;
   map<int, set<int> >::iterator rcx_map_ptr;
 
-  while(wchar_t val = readAnalysis(input))
+  wchar_t val;
+  do
   {
+    val = readAnalysis(input);
     // test for final states
     if(current_state.isFinal(all_finals))
     {
@@ -1426,7 +1630,10 @@ FSTProcessor::analysis(FILE *input, FILE *output)
 
     if(current_state.size() != 0)
     {
-      alphabet.getSymbol(sf, val);
+      if(val != 0)
+      {
+        alphabet.getSymbol(sf, val);
+      }
     }
     else
     {
@@ -1450,7 +1657,10 @@ FSTProcessor::analysis(FILE *input, FILE *output)
           {
             fputwc_unlocked(L'\\', output);
           }
-          fputwc_unlocked(val, output);
+          if(val)
+          {
+            fputwc_unlocked(val, output);
+          }
         }
       }
       else if(last_postblank)
@@ -1570,6 +1780,11 @@ FSTProcessor::analysis(FILE *input, FILE *output)
         input_buffer.setPos(last);
         input_buffer.back(1);
       }
+      if(val == 0) {
+        if(!input_buffer.isEmpty()) {
+          input_buffer.setPos(last+1);
+        }
+      }
 
       current_state = initial_state;
       lf = L"";
@@ -1579,6 +1794,7 @@ FSTProcessor::analysis(FILE *input, FILE *output)
       last_preblank = false;
     }
   }
+  while(val);
 
   // print remaining blanks
   flushBlanks(output);
@@ -2002,36 +2218,65 @@ FSTProcessor::postgeneration(FILE *input, FILE *output)
   }
 
   bool skip_mode = true;
+  collect_wblanks = false;
+  need_end_wblank = false;
   State current_state = initial_state;
   wstring lf = L"";
   wstring sf = L"";
   int last = 0;
   set<wchar_t> empty_escaped_chars;
 
-  while(wchar_t val = readPostgeneration(input))
+  while(wchar_t val = readPostgeneration(input, output))
   {
     if(val == L'~')
     {
       skip_mode = false;
+      collect_wblanks = true;
     }
 
-    if(skip_mode)
+    if(is_wblank && skip_mode)
+    {
+      //do nothing
+    }
+    else if(skip_mode)
     {
       if(iswspace(val))
       {
+        if(need_end_wblank)
+        {
+          fputws_unlocked(L"[[/]]", output);
+          need_end_wblank = false;
+        }
+
         printSpace(val, output);
       }
       else
       {
+        if(!need_end_wblank)
+        {
+          flushWblanks(output);
+        }
+
         if(isEscaped(val))
         {
           fputwc_unlocked(L'\\', output);
         }
         fputwc_unlocked(val, output);
+
+        if(need_end_wblank)
+        {
+          fputws_unlocked(L"[[/]]", output);
+          need_end_wblank = false;
+        }
       }
     }
     else
     {
+      if(is_wblank)
+      {
+        continue;
+      }
+
       // test for final states
       if(current_state.isFinal(all_finals))
       {
@@ -2108,9 +2353,14 @@ FSTProcessor::postgeneration(FILE *input, FILE *output)
       }
       else
       {
+        wstring final_wblank = combineWblanks();
+        fputws_unlocked(final_wblank.c_str(), output);
+
         if(lf == L"")
         {
           unsigned int mark = sf.size();
+          unsigned int space_index = sf.size();
+          
           for(unsigned int i = 1, limit = sf.size(); i < limit; i++)
           {
             if(sf[i] == L'~')
@@ -2118,8 +2368,36 @@ FSTProcessor::postgeneration(FILE *input, FILE *output)
               mark = i;
               break;
             }
+            else if(sf[i] == L' ')
+            {
+              space_index = i;
+            }
           }
-          fputws_unlocked(sf.substr(1, mark-1).c_str(), output);
+          
+          if(space_index != sf.size())
+          {
+            fputws_unlocked(sf.substr(1, space_index-1).c_str(), output);
+            
+            if(need_end_wblank)
+            {
+              fputws_unlocked(L"[[/]]", output);
+              need_end_wblank = false;
+              fputwc_unlocked(sf[space_index], output);
+              flushWblanks(output);
+            }
+            else
+            {
+              fputwc_unlocked(sf[space_index], output);
+            }
+            
+            fputws_unlocked(sf.substr(space_index+1, mark-space_index-1).c_str(), output);
+          }
+          else
+          {
+            flushWblanks(output);
+            fputws_unlocked(sf.substr(1, mark-1).c_str(), output);
+          }
+          
           if(mark == sf.size())
           {
             input_buffer.back(1);
@@ -2153,6 +2431,7 @@ FSTProcessor::postgeneration(FILE *input, FILE *output)
         lf = L"";
         sf = L"";
         skip_mode = true;
+        collect_wblanks = false;
       }
     }
   }
@@ -2178,7 +2457,7 @@ FSTProcessor::intergeneration(FILE *input, FILE *output)
 
   while (true)
   {
-    wchar_t val = readPostgeneration(input);
+    wchar_t val = readPostgeneration(input, output);
 
     if (val == L'~')
     {
@@ -2323,7 +2602,7 @@ FSTProcessor::transliteration(FILE *input, FILE *output)
   wstring sf = L"";
   int last = 0;
 
-  while(wchar_t val = readPostgeneration(input))
+  while(wchar_t val = readPostgeneration(input, output))
   {
     if(iswpunct(val) || iswspace(val))
     {
@@ -2803,7 +3082,7 @@ FSTProcessor::bilingual(FILE *input, FILE *output)
     symbol = tr.first;
     val = tr.second;
 
-    //fwprintf(stderr, L"> %S : %C : %d\n", tr.first.c_str(), tr.second, tr.second);
+    //fwprintf(stderr, L"> %ls : %lc : %d\n", tr.first.c_str(), tr.second, tr.second);
     if(biltransSurfaceForms && !seensurface && !outOfWord)
     {
       while(val != L'/' && val != 0x7fffffff)
@@ -2813,7 +3092,7 @@ FSTProcessor::bilingual(FILE *input, FILE *output)
         tr = readBilingual(input, output);
         symbol = tr.first;
         val = tr.second;
-        //fwprintf(stderr, L" == %S : %C : %d => %S\n", symbol.c_str(), val, val, surface.c_str());
+        //fwprintf(stderr, L" == %ls : %lc : %d => %ls\n", symbol.c_str(), val, val, surface.c_str());
       }
       seensurface = true;
       tr = readBilingual(input, output);
