@@ -16,7 +16,6 @@
  */
 
 #include <lttoolbox/expander.h>
-#include <lttoolbox/lttoolbox_config.h>
 
 #include <cstdlib>
 #include <iostream>
@@ -36,7 +35,20 @@ void endProgram(char *name)
   if(name != NULL)
   {
     cout << basename(name) << " v" << PACKAGE_VERSION <<": expand the contents of a dictionary file" << endl;
-    cout << "USAGE: " << basename(name) << " [-avlrh] dictionary_file [output_file]" << endl;
+    cout << "USAGE: " << basename(name) << " [-mavlrh] dictionary_file [output_file]" << endl;
+#if HAVE_GETOPT_LONG
+    cout << "  -m, --keep-boundaries:     keep morpheme boundaries" << endl;
+    cout << "  -v, --var:                 set language variant" << endl;
+    cout << "  -a, --alt:                 set alternative (monodix)" << endl;
+    cout << "  -l, --var-left:            set left language variant (bidix)" << endl;
+    cout << "  -r, --var-right:           set right language variant (bidix)" << endl;
+#else
+    cout << "  -m:     keep morpheme boundaries" << endl;
+    cout << "  -v:     set language variant" << endl;
+    cout << "  -a:     set alternative (monodix)" << endl;
+    cout << "  -l:     set left language variant (bidix)" << endl;
+    cout << "  -r:     set right language variant (bidix)" << endl;
+#endif
   }
   exit(EXIT_FAILURE);
 }
@@ -45,6 +57,7 @@ int main(int argc, char *argv[])
 {
   FILE *input = NULL, *output = NULL;
   Expander e;
+  e.setKeepBoundaries(false);
 
 #if HAVE_GETOPT_LONG
   int option_index=0;
@@ -54,6 +67,7 @@ int main(int argc, char *argv[])
 #if HAVE_GETOPT_LONG
     static struct option long_options[] =
     {
+      {"keep-boundaries", no_argument, 0, 'm'},
       {"alt",       required_argument, 0, 'a'},
       {"var",       required_argument, 0, 'v'},
       {"var-left",  required_argument, 0, 'l'},
@@ -62,9 +76,9 @@ int main(int argc, char *argv[])
       {0, 0, 0, 0}
     };
 
-    int cnt=getopt_long(argc, argv, "a:v:l:r:h", long_options, &option_index);
+    int cnt=getopt_long(argc, argv, "a:v:l:r:mh", long_options, &option_index);
 #else
-    int cnt=getopt(argc, argv, "a:v:l:r:h");
+    int cnt=getopt(argc, argv, "a:v:l:r:mh");
 #endif
     if (cnt==-1)
       break;
@@ -83,6 +97,10 @@ int main(int argc, char *argv[])
         e.setVariantLeftValue(optarg);
         break;
 
+      case 'm':
+        e.setKeepBoundaries(true);
+        break;
+
       case 'r':
         e.setVariantRightValue(optarg);
         break;
