@@ -49,6 +49,13 @@ private:
     bool dirty;
 
     TNodeState(Node * const &w, vector<pair<int, double>> * const &s, bool const &d): where(w), sequence(s), dirty(d){}
+
+    TNodeState(const TNodeState& other)
+      : where(other.where)
+      , sequence(other.sequence)
+      , dirty(other.dirty)
+    {}
+
     TNodeState & operator=(TNodeState const &other)
     {
       where = other.where;
@@ -65,6 +72,14 @@ private:
    */
   void destroy();
 
+  /**
+   * Helper functions for the various apply()s to reduce code duplication
+   * @return whether any transitions were made
+   */
+  bool apply_into(vector<TNodeState>* new_state, int const input, int index, bool dirty);
+
+  bool apply_into_override(vector<TNodeState>* new_state, int const input, int const old_sym, int const new_sym, int index, bool dirty);
+
   /**
    * Make a transition, version for lowercase letters and symbols
    * @param input the input symbol
@@ -103,6 +118,8 @@ private:
    */
   void apply_override(int const input, int const old_sym, int const new_sym);
 
+  void apply_override(int const input, int const alt, int const old_sym, int const new_sym);
+
   /**
    * Calculate the epsilon closure over the current state, replacing
    * its content.
@@ -179,6 +196,8 @@ public:
 
   void step_override(int const input, int const old_sym, int const new_sym);
 
+  void step_override(int const input, int const alt, int const old_sym, int const new_sym);
+
   /**
    * Init the state with the initial node and empty output
    * @param initial the initial node of the transducer
