@@ -17,7 +17,6 @@
 #include <lttoolbox/transducer.h>
 #include <lttoolbox/compression.h>
 #include <lttoolbox/alphabet.h>
-#include <lttoolbox/lttoolbox_config.h>
 #include <lttoolbox/my_stdio.h>
 #include <lttoolbox/deserialiser.h>
 #include <lttoolbox/serialiser.h>
@@ -205,25 +204,34 @@ Transducer::getInitial() const
 }
 
 set<int>
-Transducer::closure(int const state, int const epsilon_tag)
+Transducer::closure(int const state, int const epsilon_tag) const
+{
+  return closure(state, set<int>({epsilon_tag}));
+}
+
+set<int>
+Transducer::closure(int const state, set<int> const &epsilon_tags) const
 {
   set<int> nonvisited, result;
 
   nonvisited.insert(state);
   result.insert(state);
 
-  while(nonvisited.size() > 0)
-  {
+  while (nonvisited.size() > 0) {
     int auxest = *nonvisited.begin();
-    auto range = transitions[auxest].equal_range(epsilon_tag);
-    while(range.first != range.second)
-    {
-      if(result.find(range.first->second.first) == result.end())
-      {
-        result.insert(range.first->second.first);
-        nonvisited.insert(range.first->second.first);
+    for (const int epsilon_tag : epsilon_tags) {
+      try {
+        auto range = transitions.at(auxest).equal_range(epsilon_tag);
+        while (range.first != range.second) {
+          if (result.find(range.first->second.first) == result.end()) {
+            result.insert(range.first->second.first);
+            nonvisited.insert(range.first->second.first);
+          }
+          range.first++;
+        }
+      } catch (out_of_range const &e) {
+        // No transition from any of the epsilon_tags – this is fine
       }
-      range.first++;
     }
     nonvisited.erase(auxest);
   }
@@ -700,7 +708,31 @@ Transducer::reverse(int const epsilon_tag)
 }
 
 void
-Transducer::show(Alphabet const &alphabet, FILE *output, int const epsilon_tag) const
+Transducer::escapeSymbol(wstring& symbol, bool hfst) const
+{
+  if(symbol == L"") // If it's an epsilon
+  {
+    if(hfst)
+    {
+      symbol = L"@0@";
+    }
+    else
+    {
+      symbol = L"ε";
+    }
+  }
+  else if(hfst && symbol == L" ")
+  {
+    symbol = L"@_SPACE_@";
+  }
+  else if(hfst && symbol == L"\t")
+  {
+    symbol = L"@_TAB_@";
+  }
+}
+
+void
+Transducer::show(Alphabet const &alphabet, FILE *output, int const epsilon_tag, bool hfst) const
 {
   for(auto& it : transitions)
   {
@@ -711,24 +743,12 @@ Transducer::show(Alphabet const &alphabet, FILE *output, int const epsilon_tag)
       fwprintf(output, L"%d\t", it2.second.first);
       wstring l = L"";
       alphabet.getSymbol(l, t.first);
-      if(l == L"")  // If we find an epsilon
-      {
-        fwprintf(output, L"ε\t", l.c_str());
-      }
-      else
-      {
-        fwprintf(output, L"%S\t", l.c_str());
-      }
+      escapeSymbol(l, hfst);
+      fwprintf(output, L"%ls\t", l.c_str());
       wstring r = L"";
       alphabet.getSymbol(r, t.second);
-      if(r == L"")  // If we find an epsilon
-      {
-        fwprintf(output, L"ε\t", r.c_str());
-      }
-      else
-      {
-        fwprintf(output, L"%S\t", r.c_str());
-      }
+      escapeSymbol(r, hfst);
+      fwprintf(output, L"%ls\t", r.c_str());
       fwprintf(output, L"%f\t", it2.second.second);
       fwprintf(output, L"\n");
     }
@@ -741,6 +761,12 @@ Transducer::show(Alphabet const &alphabet, FILE *output, int const epsilon_tag)
   }
 }
 
+void
+Transducer::show(Alphabet const &alphabet, FILE *output, int const epsilon_tag) const
+{
+  return show(alphabet, output, epsilon_tag, false);
+}
+
 int
 Transducer::getStateSize(int const state)
 {
@@ -770,7 +796,7 @@ Transducer::recognise(wstring pattern, Alphabet &a, FILE *err)
   {
     set<int> new_state;        //Transducer::closure(int const state, int const epsilon_tag)
     // For each of the current alive states
-    //fwprintf(err, L"step: %S %C (%d)\n", pattern.c_str(), *it, sym);
+    //fwprintf(err, L"step: %ls %lc (%d)\n", pattern.c_str(), *it, sym);
     for(auto& it2 : states)
     {
       auto& p = transitions[it2];
@@ -785,7 +811,7 @@ Transducer::recognise(wstring pattern, Alphabet &a, FILE *err)
         //wstring r = L"";
         //a.getSymbol(r, t.second);
 
-        //fwprintf(err, L"  -> state: %d, trans: %S:%S, targ: %d\n", *it2, (l == L"") ?  L"ε" : l.c_str(),  (r == L"") ?  L"ε" : r.c_str(), it3->second);
+        //fwprintf(err, L"  -> state: %d, trans: %ls:%ls, targ: %d\n", *it2, (l == L"") ?  L"ε" : l.c_str(),  (r == L"") ?  L"ε" : r.c_str(), it3->second);
         //if(l.find(*it) != wstring::npos || l == L"" )
         if(l.find(it) != wstring::npos)
         {
@@ -1030,6 +1056,9 @@ Transducer::intersect(Transducer &trimmer,
   wstring compoundRSymbol = L"<compound-R>";
   wstring COMPILER_JOIN_ELEM = L"+";
   wstring COMPILER_GROUP_ELEM = L"#";
+  wstring COMPILER_ANY_TAG = L"<ANY_TAG>";
+  wstring COMPILER_ANY_CHAR = L"<ANY_CHAR>";
+  wstring COMPILER_SEPARABLE_BOUNDARY = L"<$>";
 
   // When searching, we need to record (this, (trimmer, trimmer_pre_plus))
   typedef std::pair<int, std::pair<int, int > > SearchState;
@@ -1105,7 +1134,7 @@ Transducer::intersect(Transducer &trimmer,
       wstring this_right = L"";
       this_a.getSymbol(this_right, this_a.decode(this_label).second);
 
-      if(this_right == COMPILER_JOIN_ELEM)
+      if(this_right == COMPILER_JOIN_ELEM || this_right == COMPILER_SEPARABLE_BOUNDARY)
       {
         if(trimmer_preplus == trimmer_src) {
           // Keep the old preplus state if it was set; equal to current trimmer state means unset:
@@ -1126,6 +1155,10 @@ Transducer::intersect(Transducer &trimmer,
                            trimmed_trg, // toState
                            this_label, // symbol-pair, using this alphabet
                            this_wt); //weight of transduction
+        if(this_right == COMPILER_SEPARABLE_BOUNDARY && isFinal(this_trg))
+        {
+          trimmed.setFinal(trimmed_trg, default_weight);
+        }
       }
       else if ( this_right == compoundOnlyLSymbol
                 || this_right == compoundRSymbol
@@ -1179,7 +1212,9 @@ Transducer::intersect(Transducer &trimmer,
             trimmer_preplus_next = trimmer_trg;
           }
 
-          if(trimmer_left != L"" && this_right == trimmer_left) // we've already dealt with trimmer epsilons
+          if(trimmer_left != L"" && // we've already dealt with trimmer epsilons
+             (this_right == trimmer_left ||
+              (this_right == ((trimmer_left[0] == L'<') ? COMPILER_ANY_TAG : COMPILER_ANY_CHAR))))
           {
             next = make_pair(this_trg, make_pair(trimmer_trg, trimmer_preplus_next));
             if(seen.find(next) == seen.end())
